// Load the data
const dataUrl =
  'https://gist.githubusercontent.com/RahulBelede/13d05d3379bdf5d826b41f49705b3eee/raw/9d77db0d69b1f59279c07c0f4d1b1a8981b89cd9/gistfile1.txt';

const fetchData = async () => {
  const response = await fetch(dataUrl);
  const data = await response.text();
  return d3.csvParse(data);
};

// Select the SVG from the HTML body
const svg = d3.select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');
const radius = Math.min(width, height) / 2;

// Render Pie Chart on the canvas
const render = (data) => {
  const pieData = d3.pie().value((d) => +d['2021_Sales'])(
    data,
  );

  // Colors for the sections in Pie Chart
  const sectionColors = d3.schemeCategory10;

  const colors = d3.scaleOrdinal().range(sectionColors);

  // Object for generating Arcs of the sections
  const arcGenerator = d3
    .arc()
    .innerRadius(0)
    .outerRadius(radius - 10)
    .cornerRadius(5)
    .padAngle(0.03);

  const sections = svg
    .append('g')
    .attr(
      'transform',
      `translate(${width / 2},${height / 2})`,
    )
    .selectAll('path')
    .data(pieData)
    .enter()
    .append('path')
    .attr('d', arcGenerator)
    .attr('fill', (d) => colors(d.data.Car_Type))
    .each(function (d) {
      this._current = d;
    });

  // Add car type labels with percentages beside the pie chart
  sections
    .append('text')
    .attr('transform', function (d) {
      const centroid = arcGenerator.centroid(d);
      const x =
        centroid[0] +
        (arcGenerator.outerRadius() + 10) *
          (d.endAngle < Math.PI ? 1 : -1);
      const y = centroid[1];
      return `translate(${x}, ${y})`;
    })
    .attr('dy', '.35em')
    .attr('text-anchor', function (d) {
      return d.endAngle < Math.PI ? 'start' : 'end';
    })
    .text(function (d) {
      const percentage =
        ((d.endAngle - d.startAngle) / (2 * Math.PI)) * 100;
      return `${d.data.Car_Type} (${percentage.toFixed(2)}%)`;
    });

  // Render legend for the data
  const legend = d3.select('.legend');

  const legendItems = legend
    .selectAll('.legendItem')
    .data(pieData)
    .enter()
    .append('div')
    .attr('class', 'legendItem')
    .style('display', 'flex')
    .style('align-items', 'center');

  const legendIcons = legendItems
    .append('div')
    .attr('class', 'legendIcon')
    .style('background-color', (d) =>
      colors(d.data.Car_Type),
    )
    .style('width', '18px')
    .style('height', '18px');

  legendItems
    .append('div')
    .attr('class', 'legendText')
    .text(
      (d) =>
        `${d.data.Car_Type} (${((d.endAngle - d.startAngle) / (2 * Math.PI)) * 100}%)`,
    );

  // Position legend next to the pie chart
  const legendWidth = +legend.node().offsetWidth;
  const legendHeight = +legend.node().offsetHeight;
  const legendX = width / 2 + radius + 150; // Adjust the padding as needed
  const legendY = height / 2 - legendHeight / 2;

  legend
    .style('position', 'absolute')
    .style('left', legendX + 'px')
    .style('top', legendY + 'px');
};

// Main function
const run = async () => {
  const data = await fetchData();
  render(data);
};
run();
